@@ -1,8 +1,6 @@
 #!/usr/bin/env python
 
 import os
-import sys
-import json
 import base64
 import hashlib
 import urllib.parse
@@ -11,66 +9,42 @@ import webbrowser
 from http.server import HTTPServer, BaseHTTPRequestHandler
 
 import click
-from click import pass_context, command, option, secho, echo
 import requests
 import logging
 import time
 
 from nextcode.config import get_profile_config, get_default_profile, create_profile
-from nextcode.exceptions import InvalidProfile, InvalidToken
-from nextcode.client import get_api_key
-from nextcodecli.utils import abort, dumps
+from nextcode.exceptions import InvalidProfile
+from nextcodecli.utils import abort
 
 log = logging.getLogger(__name__)
 
-def legacy_login(username, password, realm, host, is_token):
+
+def legacy_login():
     try:
         config = get_profile_config()
     except InvalidProfile as ex:
-        secho(str(ex), fg="red")
+        click.secho(str(ex), fg="red")
         abort("Please create a profile with: nextcode profile add [name]")
     profile_name = get_default_profile()
 
-    if username and password:
-        if not is_token:
-            echo("Authenticating from commandline parameters")
-        host = host or config["root_url"]
-        try:
-            api_key = get_api_key(host, username, password, realm=realm)
-        except InvalidToken as ex:
-            abort(ex)
-        if is_token:
-            click.echo(api_key)
-            return
-        create_profile(profile_name, api_key=api_key, root_url=host)
-        click.secho("Profile {} has been updated with api key".format(profile_name), bold=True)
+    root_url = config["root_url"]
+    login_server = f"{root_url.rstrip('/')}/api-key-service"
 
-    else:
-        if host:
-            root_url = "https://%s" % host
-        else:
-            root_url = config["root_url"]
-        login_server = root_url + "/api-key-service"
+    click.secho("Launching login webpage ==> Please authenticate and then press continue.", fg="yellow")
+    click.launch(login_server)
+    click.pause()
 
-        if login_server:
-            echo("Launching login webpage ==> Please authenticate and then press continue.")
-            click.launch(login_server)
-            click.pause()
-        else:
-            click.secho(
-                "No login server configured. Please aquire a refresh_token from "
-                "somewhere manually.",
-                fg='yellow',
-            )
+    # Note: readline must be imported for click.prompt() to accept long strings. Don't ask me why.
+    import readline
 
-        # Note: readline must be imported for click.prompt() to accept long strings. Don't ask me why.
-        import readline
+    api_key = click.prompt("API Key", type=str, hide_input=True)
+    try:
+        create_profile(profile_name, api_key=api_key, root_url=root_url)
+        click.secho("Profile {} has been updated with api key".format(profile_name), bold=True, fg="green")
+    except InvalidProfile as ex:
+        abort(ex)
 
-        api_key = click.prompt("API Key", type=str)
-        try:
-            create_profile(profile_name, api_key=api_key)
-        except InvalidProfile as ex:
-            abort(ex)
 
 # === PKCE Utilities ===
 def generate_pkce_pair():
@@ -91,13 +65,14 @@ class CallbackHandler(BaseHTTPRequestHandler):
 
         params = urllib.parse.parse_qs(parsed_path.query)
         self.server.auth_code = params.get("code", [None])[0]
+        self.server.state = params.get("state", [None])[0]
         self.send_response(200)
         self.send_header("Content-type", "text/html")
         self.end_headers()
         self.wfile.write(b"<html><body><h2>You may now return to your terminal.</h2></body></html>")
 
     def log_message(self, format, *args):
-        if getattr(self.server, 'verbose_logging', False):
+        if getattr(self.server, "verbose_logging", False):
             super().log_message(format, *args)
         # Otherwise suppress HTTP server logs
 
@@ -109,53 +84,60 @@ def start_server(port, verbose=False):
     return httpd
 
 
-@command()
+@click.command()
 @click.argument('tier', type=click.Choice(['dev', 'stg', 'prd']), required=False)
-@option('-u', '--username')
-@option('-p', '--password')
-@option('-r', '--realm', default='genedx-prd')
-@option(
-    '-h',
-    '--host',
-    default=None,
-    help="Host override if not using profile, e.g. platform.wuxinextcodedev.com",
-)
-@option(
+@click.option(
     '-t',
     '--token',
     'is_token',
     is_flag=True,
-    help="Return refresh token instead of writing into current profile",
+    help="Print refresh token instead of writing into current profile",
 )
-@option('-l', '--legacy', is_flag=True, help="Use legacy login")
-@option('-v', '--verbose', is_flag=True, help="Enable verbose HTTP server logging for troubleshooting")
-@pass_context
-def cli(ctx, tier, username, password, realm, host, is_token, legacy, verbose):
+@click.option('-v', '--verbose', is_flag=True, help="Enable verbose HTTP server logging for troubleshooting")
+@click.option('-l', '--legacy', is_flag=True, help="Use legacy login method")
+@click.option(
+    "--client-id",
+    default="gdb-api-key-service",
+    show_default=True,
+    help="OIDC client id",
+)
+@click.option(
+    "--scope",
+    default="openid offline_access",
+    show_default=True,
+    help="Space-delimited OAuth scope",
+)
+def cli(tier, is_token, verbose, legacy, client_id, scope):
     """
     Authenticate against keycloak.
 
-    TIER: Optional environment tier (dev|stg|prd). Defaults to 'dev'.
-    Ignored when using --legacy flag.
-
-    LEGACY: Use legacy login.
+    TIER: Optional environment tier (dev|stg|prd). Defaults to profile detected tier.
 
     VERBOSE: Enable verbose HTTP server logging for troubleshooting.
 
-    TOKEN: Return refresh token instead of writing into current profile.
+    TOKEN: Print refresh token instead of writing into current profile.
+
+    LEGACY: Use legacy login method.
     """
+    if legacy:
+        legacy_login()
+        return
+
     try:
         config = get_profile_config()
     except InvalidProfile as ex:
-        secho(str(ex), fg="red")
+        click.secho(str(ex), fg="red")
         abort("Please create a profile with: nextcode profile add [name]")
     profile_name = get_default_profile()
 
-    if legacy:
-        return legacy_login(username, password, realm, host, is_token)
-
-    # Default to dev if no tier specified
     if tier is None:
-        tier = 'dev'
+        root_url = config["root_url"]
+        if ".dev." in root_url:
+            tier = 'dev'
+        elif ".stg." in root_url:
+            tier = 'stg'
+        else:
+            tier = 'prd'
 
     configs = {
         "dev":  ("https://auth.dev.engops.genedx.net", "genedx-dev"),
@@ -163,17 +145,17 @@ def cli(ctx, tier, username, password, realm, host, is_token, legacy, verbose):
         "prd":  ("https://auth.engops.genedx.net", "genedx"),
     }
     keycloak_url, realm = configs[tier]
-    client_id = "gdb-api-key-service"
     redirect_uri = "http://localhost:8000/callback"
     port = 8000
 
     verifier, challenge = generate_pkce_pair()
+    state = base64.urlsafe_b64encode(os.urandom(32)).decode("utf-8").rstrip("=")
 
     auth_url = (
         f"{keycloak_url}/realms/{realm}/protocol/openid-connect/auth"
         f"?response_type=code&client_id={client_id}"
         f"&redirect_uri={urllib.parse.quote(redirect_uri)}"
-        f"&scope=openid&code_challenge={challenge}&code_challenge_method=S256"
+        f"&state={state}&scope={urllib.parse.quote(scope)}&code_challenge={challenge}&code_challenge_method=S256"
     )
 
     click.secho("Waiting for browser login (Ctrl+C to cancel)...", fg="green")
@@ -192,6 +174,10 @@ def cli(ctx, tier, username, password, realm, host, is_token, legacy, verbose):
         click.secho("Failed to retrieve code from callback", fg="red")
         exit(1)
 
+    if getattr(server, "state", None) != state:
+        click.secho("State mismatch; aborting.", fg="red")
+        exit(1)
+
     click.secho("Exchanging code for tokens...", fg="green")
     token_url = f"{keycloak_url}/realms/{realm}/protocol/openid-connect/token"
     response = requests.post(
@@ -207,5 +193,15 @@ def cli(ctx, tier, username, password, realm, host, is_token, legacy, verbose):
     )
 
     tokens = response.json()
+    api_key = tokens['refresh_token']
+
+    if is_token:
+        click.echo(api_key)
+        return
+
+    try:
+        create_profile(profile_name, api_key=api_key, root_url=config["root_url"])
+        click.secho("Profile {} has been updated with api key".format(profile_name), bold=True, fg="green")
+    except InvalidProfile as ex:
+        abort(ex)
 
-    click.echo(json.dumps(tokens, indent=2))
